"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getContainerXPath = getContainerXPath;
exports.getPureHTMLStringWithoutScripts = getPureHTMLStringWithoutScripts;
exports.performanceGetEntriesByName = performanceGetEntriesByName;
exports.performanceMark = performanceMark;
exports.performanceMeasure = performanceMeasure;
exports.toArray = toArray;
var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));
var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));
/**
 * @author Kuitos
 * @since 2023-04-25
 */
function toArray(array) {
  return Array.isArray(array) ? array : [array];
}

/**
 * copy from https://developer.mozilla.org/zh-CN/docs/Using_XPath
 * @param el
 * @param document
 */
function getXPathForElement(el, document) {
  // not support that if el not existed in document yet(such as it not append to document before it mounted)
  if (!document.body.contains(el)) {
    return undefined;
  }
  var xpath = '';
  var pos;
  var tmpEle;
  var element = el;
  while (element !== document.documentElement) {
    pos = 0;
    tmpEle = element;
    while (tmpEle) {
      if (tmpEle.nodeType === 1 && tmpEle.nodeName === element.nodeName) {
        // If it is ELEMENT_NODE of the same name
        pos += 1;
      }
      tmpEle = tmpEle.previousSibling;
    }
    xpath = "*[name()='".concat(element.nodeName, "'][").concat(pos, "]/").concat(xpath);
    element = element.parentNode;
  }
  xpath = "/*[name()='".concat(document.documentElement.nodeName, "']/").concat(xpath);
  xpath = xpath.replace(/\/$/, '');
  return xpath;
}
function getContainerXPath(container) {
  return getXPathForElement(container, document);
}
var supportsUserTiming = typeof performance !== 'undefined' && typeof performance.mark === 'function' && typeof performance.clearMarks === 'function' && typeof performance.measure === 'function' && typeof performance.clearMeasures === 'function' && typeof performance.getEntriesByName === 'function';
function performanceGetEntriesByName(markName, type) {
  var marks = null;
  if (supportsUserTiming) {
    marks = performance.getEntriesByName(markName, type);
  }
  return marks;
}
function performanceMark(markName) {
  if (supportsUserTiming) {
    performance.mark(markName);
  }
}
function performanceMeasure(measureName, markName) {
  if (supportsUserTiming && performance.getEntriesByName(markName, 'mark').length) {
    performance.measure(measureName, markName);
    performance.clearMarks(markName);
    performance.clearMeasures(measureName);
  }
}
function getPureHTMLStringWithoutScripts(_x, _x2) {
  return _getPureHTMLStringWithoutScripts.apply(this, arguments);
}
function _getPureHTMLStringWithoutScripts() {
  _getPureHTMLStringWithoutScripts = (0, _asyncToGenerator2.default)( /*#__PURE__*/_regenerator.default.mark(function _callee(entry, fetch) {
    var htmlString, domParser, htmlDOM;
    return _regenerator.default.wrap(function _callee$(_context) {
      while (1) switch (_context.prev = _context.next) {
        case 0:
          _context.next = 2;
          return fetch(entry).then(function (r) {
            return r.text();
          });
        case 2:
          htmlString = _context.sent;
          domParser = new DOMParser();
          htmlDOM = domParser.parseFromString(htmlString, 'text/html'); // remove all script tags who are been loaded before
          htmlDOM.querySelectorAll('script').forEach(function (script) {
            return script.remove();
          });
          return _context.abrupt("return", htmlDOM.documentElement.outerHTML);
        case 7:
        case "end":
          return _context.stop();
      }
    }, _callee);
  }));
  return _getPureHTMLStringWithoutScripts.apply(this, arguments);
}
//# sourceMappingURL=utils.js.map