import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import _asyncToGenerator from "@babel/runtime/helpers/esm/asyncToGenerator";
import _toConsumableArray from "@babel/runtime/helpers/esm/toConsumableArray";
import _regeneratorRuntime from "@babel/runtime/regenerator";
import { mountRootParcel } from 'single-spa';
import loadApp from "../core/loadApp";
import { getContainerXPath, toArray } from "../utils";
import { start, started } from "./registerMicroApps";
var appConfigPromiseGetterMap = new Map();
var containerMicroAppsMap = new Map();
export function loadMicroApp(app, configuration, lifeCycles) {
  var props = app.props,
    name = app.name,
    container = app.container;

  // Must compute the container xpath at beginning to keep it consist around app running
  // If we compute it every time, the container dom structure most probably been changed and result in a different xpath value
  var containerXPath = getContainerXPath(container);
  var getContainerXPathKey = function getContainerXPathKey(xpath) {
    return "".concat(name, "-").concat(xpath);
  };
  var microApp;
  var wrapParcelConfigForRemount = function wrapParcelConfigForRemount(config) {
    var microAppConfig = config;
    if (containerXPath) {
      var appContainerXPathKey = getContainerXPathKey(containerXPath);
      var containerMicroApps = containerMicroAppsMap.get(appContainerXPathKey);
      if (containerMicroApps !== null && containerMicroApps !== void 0 && containerMicroApps.length) {
        var mount = [/*#__PURE__*/_asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee() {
          var prevLoadMicroApps, prevLoadMicroAppsWhichNotBroken;
          return _regeneratorRuntime.wrap(function _callee$(_context) {
            while (1) switch (_context.prev = _context.next) {
              case 0:
                // While there are multiple micro apps mounted on the same container, we must wait until the prev instances all had unmounted
                // Otherwise it will lead some concurrent issues
                prevLoadMicroApps = containerMicroApps.slice(0, containerMicroApps.indexOf(microApp));
                prevLoadMicroAppsWhichNotBroken = prevLoadMicroApps.filter(function (v) {
                  return v.getStatus() !== 'LOAD_ERROR' && v.getStatus() !== 'SKIP_BECAUSE_BROKEN';
                });
                _context.next = 4;
                return Promise.all(prevLoadMicroAppsWhichNotBroken.map(function (v) {
                  return v.unmountPromise;
                }));
              case 4:
              case "end":
                return _context.stop();
            }
          }, _callee);
        }))].concat(_toConsumableArray(toArray(microAppConfig.mount)));
        microAppConfig = _objectSpread(_objectSpread({}, config), {}, {
          mount: mount
        });
      }
    }
    return _objectSpread(_objectSpread({}, microAppConfig), {}, {
      // empty bootstrap hook which should not run twice while it calling from cached micro app
      bootstrap: function bootstrap() {
        return Promise.resolve();
      }
    });
  };

  /**
   * using name + container xpath as the micro app instance id,
   * it means if you're rendering a micro app to a dom which have been rendered before,
   * the micro app would not load and evaluate its lifecycles again
   */
  var memorizedLoadingFn = /*#__PURE__*/function () {
    var _ref2 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee2() {
      var userConfiguration, appContainerXPathKey, parcelConfigGetterPromise, parcelConfigObjectGetterPromise, _appContainerXPathKey;
      return _regeneratorRuntime.wrap(function _callee2$(_context2) {
        while (1) switch (_context2.prev = _context2.next) {
          case 0:
            userConfiguration = configuration;
            if (!containerXPath) {
              _context2.next = 11;
              break;
            }
            appContainerXPathKey = getContainerXPathKey(containerXPath);
            parcelConfigGetterPromise = appConfigPromiseGetterMap.get(appContainerXPathKey);
            if (!parcelConfigGetterPromise) {
              _context2.next = 11;
              break;
            }
            _context2.t0 = wrapParcelConfigForRemount;
            _context2.next = 8;
            return parcelConfigGetterPromise;
          case 8:
            _context2.t1 = _context2.sent;
            _context2.t2 = (0, _context2.t1)(container);
            return _context2.abrupt("return", (0, _context2.t0)(_context2.t2));
          case 11:
            parcelConfigObjectGetterPromise = loadApp(app, userConfiguration, lifeCycles);
            if (containerXPath) {
              _appContainerXPathKey = "".concat(name, "-").concat(containerXPath);
              appConfigPromiseGetterMap.set(_appContainerXPathKey, parcelConfigObjectGetterPromise);
            }
            _context2.next = 15;
            return parcelConfigObjectGetterPromise;
          case 15:
            _context2.t3 = _context2.sent;
            return _context2.abrupt("return", (0, _context2.t3)(container));
          case 17:
          case "end":
            return _context2.stop();
        }
      }, _callee2);
    }));
    return function memorizedLoadingFn() {
      return _ref2.apply(this, arguments);
    };
  }();
  if (!started) {
    // We need to invoke start method of single-spa as the popstate event should be dispatched while the main app calling pushState/replaceState automatically,
    // but in single-spa it will check the start status before it dispatch popstate
    // see https://github.com/single-spa/single-spa/blob/f28b5963be1484583a072c8145ac0b5a28d91235/src/navigation/navigation-events.js#L101
    // ref https://github.com/umijs/qiankun/pull/1071
    start();
  }
  microApp = mountRootParcel(memorizedLoadingFn, _objectSpread({
    domElement: document.createElement('div')
  }, props));
  if (containerXPath) {
    var appContainerXPathKey = getContainerXPathKey(containerXPath);
    // Store the microApps which they mounted on the same container
    var microAppsRef = containerMicroAppsMap.get(appContainerXPathKey) || [];
    microAppsRef.push(microApp);
    containerMicroAppsMap.set(appContainerXPathKey, microAppsRef);
    var cleanup = function cleanup() {
      var index = microAppsRef.indexOf(microApp);
      microAppsRef.splice(index, 1);
      // eslint-disable-next-line @typescript-eslint/ban-ts-comment
      // @ts-ignore
      microApp = null;
    };

    // gc after unmount
    microApp.unmountPromise.then(cleanup).catch(cleanup);
  }
  return microApp;
}
//# sourceMappingURL=loadMicroApp.js.map